{***************************************************************************}
{                       UNIDAD DE VIDEO GRAPHIC ARRAY

                                  Copyright (c) 1998 Marco Antonio Alvarado
}
{***************************************************************************}

UNIT UVGA;

INTERFACE

  CONST
    AnchoPantalla = 320;
    AltoPantalla = 200;
    TamanoPantalla = AnchoPantalla * AltoPantalla;
    X1Pantalla = 0;
    Y1Pantalla = 0;
    X2Pantalla = AnchoPantalla - 1;
    Y2Pantalla = AltoPantalla - 1;
    XCentroPantalla = (AnchoPantalla DIV 2) - 1;
    YCentroPantalla = (AltoPantalla DIV 2) - 1;

  TYPE
    PPantalla = ^APantalla;
    APantalla = ARRAY [0..TamanoPantalla - 1] OF Byte;
    PMezcla3 = ^RMezcla3;

    RMezcla3 = RECORD
      Rojo,
      Verde,
      Azul : Byte;
    END;

    PPaleta = ^APaleta;
    APaleta = ARRAY [Byte] OF RMezcla3;

  VAR
{Error:
  1 = el procesador no es 80386 o superior.
  2 = no puede ajustar el modo VGA.}
    Error : Word;

    Previa : PPantalla;

  PROCEDURE Espera;
  FUNCTION CopiaPunto (Punto : Word) : Byte;
  PROCEDURE PegaPunto (Color : Byte; Punto : Word);
  PROCEDURE LlenaPunto (Color : Byte; Punto, Cantidad : Word);
  PROCEDURE CopiaPintura (Punto : Word; Pintura : Pointer; Cantidad : Word);
  PROCEDURE PegaPintura (Pintura : Pointer; Punto, Cantidad : Word);
  PROCEDURE PegaPinturaHueco0 (Pintura : Pointer; Punto, Cantidad : Word);
  PROCEDURE PegaPinturaHueco (Pintura : Pointer; Punto, Cantidad : Word;
    Color : Byte);
  PROCEDURE PegaPinturaSolo (Pintura : Pointer; Punto, Cantidad : Word;
    Color : Byte);
  PROCEDURE BorraPantalla;
  PROCEDURE CopiaPantalla (Pantalla : PPantalla);
  PROCEDURE PegaPantalla (Pantalla : PPantalla);

  PROCEDURE BorraPrevia;
  PROCEDURE LlenaPrevia (Color : Byte);
  PROCEDURE PegaPrevia;
  FUNCTION CopiaPuntoPrevia (Punto : Word) : Byte;
  PROCEDURE PegaPuntoPrevia (Color : Byte; Punto : Word);
  PROCEDURE LlenaPuntoPrevia (Color : Byte; Punto, Cantidad : Word);
  PROCEDURE CopiaPinturaPrevia (Punto : Word; Pintura : Pointer; Cantidad :
    Word);
  PROCEDURE PegaPinturaPrevia (Pintura : Pointer; Punto, Cantidad : Word);
  PROCEDURE PegaPinturaHueco0Previa (Pintura : Pointer; Punto, Cantidad :
    Word);
  PROCEDURE PegaPinturaHuecoPrevia (Pintura : Pointer; Punto, Cantidad :
    Word; Color : Byte);
  PROCEDURE PegaPinturaSoloPrevia (Pintura : Pointer; Punto, Cantidad : Word;
    Color : Byte);

  PROCEDURE LlenaIntensidad (Intensidad : Byte; Color : Byte; Cantidad :
    Word);
  PROCEDURE CopiaMezcla (Color : Byte; Mezcla : Pointer; Cantidad : Word);
  PROCEDURE PegaMezcla (Mezcla : Pointer; Color : Byte; Cantidad : Word);
  PROCEDURE BorraPaleta;
  PROCEDURE CopiaPaleta (Paleta : PPaleta);
  PROCEDURE PegaPaleta (Paleta : PPaleta);

IMPLEMENTATION

  USES
    UVBS;  {Unidad de Video BIOS Services}

  CONST
    SegmentoPantalla = $a000;
    TamanoOperando = $66;
    Modo = $13;

  VAR
    ViejoModo : Byte;
    ViejoExitProc : Pointer;

  PROCEDURE Espera; ASSEMBLER;

    LABEL
      Ciclo1, Ciclo2;

  ASM
    MOV DX, 3dah

    Ciclo1:

    IN AL, DX
    AND AL, 8
    JNZ Ciclo1

    Ciclo2:

    IN AL, DX
    AND AL, 8
    JZ Ciclo2
  END;

  FUNCTION CopiaPunto (Punto : Word) : Byte; ASSEMBLER;
  ASM
{Apunta a la direccin del Punto.}
    MOV BX, SegmentoPantalla
    MOV ES, BX
    MOV BX, Punto

{Copia el Color.}
    SEGES MOV AL, [BX]
  END;

  PROCEDURE PegaPunto (Color : Byte; Punto : Word); ASSEMBLER;
  ASM
{Apunta a la direccin del Punto.}
    MOV BX, SegmentoPantalla
    MOV ES, BX
    MOV BX, Punto

{Pega el Color.}
    MOV AL, Color
    SEGES MOV [BX], AL
  END;

  PROCEDURE LlenaPunto (Color : Byte; Punto, Cantidad : Word); ASSEMBLER;
  ASM
{Forma un Color de cuatro letras en el registro EAX.}
    MOV AL, Color
    MOV AH, AL
    MOV DX, AX
    DB TamanoOperando
    SHL AX, 16
    MOV AX, DX

{Apunta a la direccin del Punto como destino.}
    MOV DI, SegmentoPantalla
    MOV ES, DI
    MOV DI, Punto

{Resuelve la Cantidad de dobles palabras a Pegar.}
    MOV CX, Cantidad
    MOV DX, CX
    AND DX, 3
    SHR CX, 2

{Pega las dobles palabras.}
    DB TamanoOperando
    REP STOSW

{Pega las letras que faltan.}
    MOV CX, DX
    REP STOSB
  END;

  PROCEDURE CopiaPintura (Punto : Word; Pintura : Pointer; Cantidad : Word);
    ASSEMBLER;
  ASM
{Salva el registro DS para que no se pierdan las variables.}
    PUSH DS

{Apunta AL destino.}
    LES DI, Pintura

{Resuelve la Cantidad de dobles palabras a Copiar.}
    MOV CX, Cantidad
    MOV DX, CX
    AND DX, 3
    SHR CX, 2

{Apunta a la direccin del Punto como fuente.}
    MOV BX, Punto
    MOV SI, SegmentoPantalla
    MOV DS, SI
    MOV SI, BX

{Copia las dobles palabras.}
    DB TamanoOperando
    REP MOVSW

{Copia las letras que faltan.}
    MOV CX, DX
    REP MOVSB

{Repone DS.}
    POP DS
  END;

  PROCEDURE PegaPintura (Pintura : Pointer; Punto, Cantidad : Word);
    ASSEMBLER;
  ASM
{Salva el registro DS para que no se pierdan las variables.}
    PUSH DS

{Apunta a la direccin del Punto como destino.}
    MOV DI, SegmentoPantalla
    MOV ES, DI
    MOV DI, Punto

{Resuelve la Cantidad de dobles palabras a Pegar.}
    MOV CX, Cantidad
    MOV DX, CX
    AND DX, 3
    SHR CX, 2

{Apunta a la fuente.}
    LDS SI, Pintura

{Pega las dobles palabras.}
    DB TamanoOperando
    REP MOVSW

{Pega las letras que faltan.}
    MOV CX, DX
    REP MOVSB

{Repone DS.}
    POP DS
  END;

  PROCEDURE PegaPinturaHueco0 (Pintura : Pointer; Punto, Cantidad : Word);
    ASSEMBLER;

    LABEL
      Ciclo, Siguiente, Fin;

  ASM
{Salva el registro DS para que no se pierdan las variables.}
    PUSH DS

{Carga la Cantidad.}
    MOV CX, Cantidad

{Apunta a la direccin del Punto como destino.}
    MOV DI, SegmentoPantalla
    MOV ES, DI
    MOV DI, Punto

{Apunta a la fuente.}
    LDS SI, Pintura

    Ciclo:

    LODSB
    TEST AL, AL
    JZ Siguiente
    STOSB
    LOOP Ciclo
    JMP Fin

    Siguiente:

    INC DI
    LOOP Ciclo

    Fin:

{Repone DS.}
    POP DS
  END;

  PROCEDURE PegaPinturaHueco (Pintura : Pointer; Punto, Cantidad : Word;
    Color : Byte); ASSEMBLER;

    LABEL
      Ciclo, Siguiente, Fin;

  ASM
{Salva el registro DS para que no se pierdan las variables.}
    PUSH DS

{Carga la Cantidad.}
    MOV CX, Cantidad

{Carga el Color a excluir.}
    MOV DL, Color

{Apunta a la direccin del Punto como destino.}
    MOV DI, SegmentoPantalla
    MOV ES, DI
    MOV DI, Punto

{Apunta a la fuente.}
    LDS SI, Pintura

    Ciclo:

    LODSB
    CMP AL, DL
    JE Siguiente
    STOSB
    LOOP Ciclo
    JMP Fin

    Siguiente:

    INC DI
    LOOP Ciclo

    Fin:

{Repone DS.}
    POP DS
  END;

  PROCEDURE PegaPinturaSolo (Pintura : Pointer; Punto, Cantidad : Word;
    Color : Byte); ASSEMBLER;

    LABEL
      Ciclo, Siguiente, Fin;

  ASM
{Salva el registro DS para que no se pierdan las variables.}
    PUSH DS

{Carga la Cantidad.}
    MOV CX, Cantidad

{Carga el Color que excluye.}
    MOV DL, Color

{Apnta a la direccin del Punto como destino.}
    MOV DI, SegmentoPantalla
    MOV ES, DI
    MOV DI, Punto

{Apunta a la fuente.}
    LDS SI, Pintura

    Ciclo:

    LODSB
    CMP AL, DL
    JNE Siguiente
    STOSB
    LOOP Ciclo
    JMP Fin

    Siguiente:

    INC DI
    LOOP Ciclo

    Fin:

{Repone DS.}
    POP DS
  END;

  PROCEDURE BorraPantalla; ASSEMBLER;
  ASM
{Carga 0 en el registro EAX.}
    DB TamanoOperando
    XOR AX, AX

{Apunta AL inicio de la Pantalla como destino.}
    MOV DI, SegmentoPantalla
    MOV ES, DI
    XOR DI, DI

{Carga la Cantidad de dobles palabras a Pegar.}
    MOV CX, 16000

{Pega las dobles palabras.}
    DB TamanoOperando
    REP STOSW
  END;

  PROCEDURE CopiaPantalla (Pantalla : PPantalla); ASSEMBLER;
  ASM
{Salva DS para que no se pierdan las variables.}
    PUSH DS

{Apunta AL destino.}
    LES DI, Pantalla

{Apunta AL inicio de la Pantalla como fuente.}
    MOV SI, SegmentoPantalla
    MOV DS, SI
    XOR SI, SI

{Carga la Cantidad de dobles palabras a Copiar.}
    MOV CX, 16000

{Copia las dobles palabras.}
    DB TamanoOperando
    REP MOVSW

{Repone DS.}
    POP DS
  END;

  PROCEDURE PegaPantalla (Pantalla : PPantalla); ASSEMBLER;
  ASM
{Salva DS para que no se pierdan las variables.}
    PUSH DS

{Apunta AL inicio de la Pantalla como destino.}
    MOV DI, SegmentoPantalla
    MOV ES, DI
    XOR DI, DI

{Apunta a la fuente.}
    LDS SI, Pantalla

{Carga la Cantidad de dobles palabras a Pegar.}
    MOV CX, 16000

{Pega las dobles palabras.}
    DB TamanoOperando
    REP MOVSW

{Repone DS.}
    POP DS
  END;

  PROCEDURE BorraPrevia; ASSEMBLER;
  ASM
{Carga 0 en el registro EAX.}
    DB TamanoOperando
    XOR AX, AX

{Apunta AL inicio de la Pantalla como destino.}
    LES DI, Previa

{Carga la Cantidad de dobles palabras a Pegar.}
    MOV CX, 16000

{Pega las dobles palabras.}
    DB TamanoOperando
    REP STOSW
  END;

  PROCEDURE LlenaPrevia (Color : Byte); ASSEMBLER;
  ASM
{Carga el Color en el registro EAX.}
    MOV AL, Color
    MOV AH, AL
    MOV DX, AX
    DB TamanoOperando
    SHL AX, 16
    MOV AX, DX

{Apunta AL inicio de la Pantalla como destino.}
    LES DI, Previa

{Carga la Cantidad de dobles palabras a Pegar.}
    MOV CX, 16000

{Pega las dobles palabras.}
    DB TamanoOperando
    REP STOSW
  END;

  PROCEDURE PegaPrevia;
  BEGIN
    Espera;
    PegaPantalla (Previa);
  END;

  FUNCTION CopiaPuntoPrevia (Punto : Word) : Byte; ASSEMBLER;
  ASM
{Apunta a la direccin del Punto.}
    LES BX, Previa
    ADD BX, Punto

{Copia el Color.}
    SEGES MOV AL, [BX]
  END;

  PROCEDURE PegaPuntoPrevia (Color : Byte; Punto : Word); ASSEMBLER;
  ASM
{Apunta a la direccin del Punto.}
    LES BX, Previa
    ADD BX, Punto

{Pega el Color.}
    MOV AL, Color
    SEGES MOV [BX], AL
  END;

  PROCEDURE LlenaPuntoPrevia (Color : Byte; Punto, Cantidad : Word);
    ASSEMBLER;
  ASM
{Forma un Color de cuatro letras en el registro EAX.}
    MOV AL, Color
    MOV AH, AL
    MOV DX, AX
    DB TamanoOperando
    SHL AX, 16
    MOV AX, DX

{Apunta a la direccin del Punto como destino.}
    LES DI, Previa
    ADD DI, Punto

{Resuelve la Cantidad de dobles palabras a Pegar.}
    MOV CX, Cantidad
    MOV DX, CX
    AND DX, 3
    SHR CX, 2

{Pega las dobles palabras.}
    DB TamanoOperando
    REP STOSW

{Pega las letras que faltan.}
    MOV CX, DX
    REP STOSB
  END;

  PROCEDURE CopiaPinturaPrevia (Punto : Word; Pintura : Pointer; Cantidad :
    Word); ASSEMBLER;
  ASM
{Salva el registro DS para que no se pierdan las variables.}
    PUSH DS

{Apunta AL destino.}
    LES DI, Pintura

{Resuelve la Cantidad de dobles palabras a Copiar.}
    MOV CX, Cantidad
    MOV DX, CX
    AND DX, 3
    SHR CX, 2

{Apunta a la direccin del Punto como fuente.}
    MOV BX, Punto
    LDS SI, Previa
    ADD SI, BX

{Copia las dobles palabras.}
    DB TamanoOperando
    REP MOVSW

{Copia las letras que faltan.}
    MOV CX, DX
    REP MOVSB

{Repone DS.}
    POP DS
  END;

  PROCEDURE PegaPinturaPrevia (Pintura : Pointer; Punto, Cantidad : Word);
    ASSEMBLER;
  ASM
{Salva el registro DS para que no se pierdan las variables.}
    PUSH DS

{Apunta a la direccin del Punto como destino.}
    LES DI, Previa
    ADD DI, Punto

{Resuelve la Cantidad de dobles palabras a Pegar.}
    MOV CX, Cantidad
    MOV DX, CX
    AND DX, 3
    SHR CX, 2

{Apunta a la fuente.}
    LDS SI, Pintura

{Pega las dobles palabras.}
    DB TamanoOperando
    REP MOVSW

{Pega las letras que faltan.}
    MOV CX, DX
    REP MOVSB

{Repone DS.}
    POP DS
  END;

  PROCEDURE PegaPinturaHueco0Previa (Pintura : Pointer; Punto, Cantidad :
    Word); ASSEMBLER;

    LABEL
      Ciclo, Siguiente, Fin;

  ASM
{Salva el registro DS para que no se pierdan las variables.}
    PUSH DS

{Carga la Cantidad.}
    MOV CX, Cantidad

{Apunta a la direccin del Punto como destino.}
    LES DI, Previa
    ADD DI, Punto

{Apunta a la fuente.}
    LDS SI, Pintura

    Ciclo:

    LODSB
    TEST AL, AL
    JZ Siguiente
    STOSB
    LOOP Ciclo
    JMP Fin

    Siguiente:

    INC DI
    LOOP Ciclo

    Fin:

{Repone DS.}
    POP DS
  END;

  PROCEDURE PegaPinturaHuecoPrevia (Pintura : Pointer; Punto, Cantidad :
    Word; Color : Byte); ASSEMBLER;

    LABEL
      Ciclo, Siguiente, Fin;

  ASM
{Salva el registro DS para que no se pierdan las variables.}
    PUSH DS

{Carga la Cantidad.}
    MOV CX, Cantidad

{Carga el Color a excluir.}
    MOV DL, Color

{Apunta a la direccin del Punto como destino.}
    LES DI, Previa
    ADD DI, Punto

{Apunta a la fuente.}
    LDS SI, Pintura

    Ciclo:

    LODSB
    CMP AL, DL
    JE Siguiente
    STOSB
    LOOP Ciclo
    JMP Fin

    Siguiente:

    INC DI
    LOOP Ciclo

    Fin:

{Repone DS.}
    POP DS
  END;

  PROCEDURE PegaPinturaSoloPrevia (Pintura : Pointer; Punto, Cantidad : Word;
    Color : Byte); ASSEMBLER;

    LABEL
      Ciclo, Siguiente, Fin;

  ASM
{Salva el registro DS para que no se pierdan las variables.}
    PUSH DS

{Carga la Cantidad.}
    MOV CX, Cantidad

{Carga el Color que excluye.}
    MOV DL, Color

{Apnta a la direccin del Punto como destino.}
    LES DI, Previa
    ADD DI, Punto

{Apunta a la fuente.}
    LDS SI, Pintura

    Ciclo:

    LODSB
    CMP AL, DL
    JNE Siguiente
    STOSB
    LOOP Ciclo
    JMP Fin

    Siguiente:

    INC DI
    LOOP Ciclo

    Fin:

{Repone DS.}
    POP DS
  END;

  PROCEDURE LlenaIntensidad (Intensidad : Byte; Color : Byte; Cantidad :
    Word); ASSEMBLER;

    LABEL
      Ciclo;

  ASM
    MOV bl, Intensidad
    MOV AL, Color
    MOV CX, Cantidad
    MOV DX, 3c8h
    OUT DX, AL
    MOV DX, 3c9h
    MOV AL, bl

    Ciclo:

    OUT DX, AL
    LOOP Ciclo
  END;

  PROCEDURE CopiaMezcla (Color : Byte; Mezcla : Pointer; Cantidad : Word);
    ASSEMBLER;

    LABEL
      Ciclo;

  ASM
    MOV AL, Color
    MOV CX, Cantidad
    LES DI, Mezcla
    MOV DX, 3c7h
    OUT DX, AL
    MOV DX, 3c9h

    Ciclo:

    INSB
    LOOP Ciclo
  END;

  PROCEDURE PegaMezcla (Mezcla : Pointer; Color : Byte; Cantidad : Word);
    ASSEMBLER;

    LABEL
      Ciclo;

  ASM
    MOV AL, Color
    MOV CX, Cantidad
    LES SI, Mezcla
    MOV DX, 3c8h
    OUT DX, AL
    MOV DX, 3c9h

    Ciclo:

    SEGES outsb
    LOOP Ciclo
  END;

  PROCEDURE BorraPaleta; ASSEMBLER;

    LABEL
      Ciclo;

  ASM
    XOR AL, AL
    MOV CX, 768
    MOV DX, 3c8h
    OUT DX, AL
    MOV DX, 3c9h

    Ciclo:

    OUT DX, AL
    LOOP Ciclo
  END;

  PROCEDURE CopiaPaleta (Paleta : PPaleta); ASSEMBLER;

    LABEL
      Ciclo;

  ASM
    XOR AL, AL
    MOV CX, 768
    LES DI, Paleta
    MOV DX, 3c7h
    OUT DX, AL
    MOV DX, 3c9h

    Ciclo:

    INSB
    LOOP Ciclo
  END;

  PROCEDURE PegaPaleta (Paleta : PPaleta); ASSEMBLER;

    LABEL
      Ciclo;

  ASM
    XOR AL, AL
    MOV CX, 768
    LES SI, Paleta
    MOV DX, 3c8h
    OUT DX, AL
    MOV DX, 3c9h

    Ciclo:

    SEGES outsb
    LOOP Ciclo
  END;

{$F+}
  PROCEDURE Sale;
{$F-}
  BEGIN
    ExitProc := ViejoExitProc;

    Dispose (Previa);
    Espera;
    PoneModo (ViejoModo);
  END;

BEGIN
  Error := 0;

  IF Test8086 < 2 THEN
  BEGIN
    Error := 1;
  END;

  ViejoModo := UVBS. ModoActual;
  Espera;
  UVBS. PoneModo (Modo);

  IF UVBS. ModoActual <> Modo THEN
  BEGIN
    Error := 2;
  END;

  New (Previa);
  BorraPrevia;

  ViejoExitProc := ExitProc;
  ExitProc := @Sale;
END.